@@ -1,5 +1,5 @@
 # $File: //member/autrijus/DBIx-ReportBuilder/lib/DBIx/ReportBuilder/Search.pm $ $Author: autrijus $
-# $Revision: #1 $ $Change: 7953 $ $DateTime: 2003/09/07 22:05:43 $
+# $Revision: #2 $ $Change: 8037 $ $DateTime: 2003/09/10 18:33:37 $
 
 package DBIx::ReportBuilder::Search;
 
@@ -8,6 +8,7 @@ use base 'DBIx::SearchBuilder';
 
 my $stub;
 sub NewItem { bless(\$stub) }
-sub LoadFromHash { $_[0] = $_[1] }
+sub LoadFromHash { bless($_[0] = $_[1]) }
+sub Id { $_[0]->{id} }
 
 1;
